//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
#[cfg(feature = "objc2-core-foundation")]
use objc2_core_foundation::*;

use crate::*;

#[cfg(feature = "objc2-core-foundation")]
unsafe impl ConcreteType for ODContextRef {
    /// Standard GetTypeID function support for CF-based objects
    ///
    /// Returns the typeID for the ODContext object
    ///
    /// Returns: a valid CFTypeID for the ODContext object
    #[doc(alias = "ODContextGetTypeID")]
    #[inline]
    fn type_id() -> CFTypeID {
        extern "C-unwind" {
            fn ODContextGetTypeID() -> CFTypeID;
        }
        unsafe { ODContextGetTypeID() }
    }
}
